%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% KOMPENSATIONS-REGLERENTWURF
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
clear all;
close all;

%Strecke
s = tf('s');  
G = 222.22/(1+s/11.7)/(1+s/1321.66)/(1+2*0.138*s/1.82+(s/1.82)^2)

%Anforderungen an die Sprungantwort des geschlossenen Regelkreises
tr = 0.5;
ue = 5;

%Anforderungen an das Bodediagramm des offenen Regelkreises
wc = 1.5/tr;
phi = 70-ue;

%% Kompensationsregler mit Integralanteil
R1 = (1+2*0.138*s/1.82+(s/1.82)^2)/s;

%Bodediagramm von L1
L1 = R1*G;
figure(1);
bode(L1);
grid on;

%Betrags- und Phasenkorrektur
L1_wc = freqresp(L1,wc);
dphi = -(180-phi)*pi/180-(angle(L1_wc));
Tr = tan(-dphi/2)/wc;

L2 = L1/(1+s*Tr)^2;
L2_wc = freqresp(L2,wc);
Vr = 1/abs(L2_wc);

%Bodediagramm des offenen Regelkreises
L = Vr*L2;
figure(2);
bode(L1,L);
grid on;

%Test, dass Betrag und Phase an der Durchtrittsfrequenz korrekt sind 
disp(['Phase bei Durchtrittsfrequenz: ',num2str(angle(freqresp(L,wc))*180/pi)]);
disp(['Amplitude bei Durchtrittsfrequenz: ',num2str(abs(freqresp(L,wc)))]);

%Regler
R = Vr*R1/(1+s*Tr)^2

%Sprungantwort des geschlossenen Regelkreises
T = feedback(L,1);
figure(3);
step(T,3);
grid on;

%Sprungantwort der Stellgrenbertragungsfunktion
Tu = R/(1+R*G);
figure(4);
step(100*Tu,3);
grid on;

%Vorfilter
F = 1/((s/5)^2+2*0.92*s/5+1);
t = 0:0.001:3;

%Sprungantworten mit Vorfilter
Tu1 = F*R/(1+R*G);
T1 = Tu1*G;
figure(5);
step(100*Tu,t);
hold on;
step(100*Tu1,t);
grid on;
grid on;
figure(6);
step(T,t);
hold on;
step(T1,t);
grid on;
